SystemOrganization addCategory: #'UnixMetaStack-Profiling'!

StackInterpreterPrimitives subclass: #ProfileStackInterpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

ProfileStackInterpreter subclass: #ExtendedProfileStackInterpreter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:37'!
fetchNextBytecode

	"This method fetches the next instruction (bytecode). Each bytecode method is responsible for fetching the next bytecode, preferably as early as possible to allow the memory system time to process the request before the next dispatch."
	
	"bytecode counters -- for analysis see p. 595 of blue book and StackInterpreter>>initializeBytecodeTableForSqueakV3PlusClosures"
	self incByteCodeCounter.
	self cppIf: MULTIPLEBYTECODESETS
		ifTrue: [currentBytecode := self fetchByte + bytecodeSetSelector]
		ifFalse: [currentBytecode := self fetchByte].! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:37'!
incByteCodeCounter

	<api>
	self cCode: 'bCounter++;bArrayCounter[getCurrentBytecode()]++;'! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:19'!
internalExecuteNewMethod
	"regardless of whether it succeeded or not count it since it is executed"
	"for primitive Stats check StackInterpreter>>initializePrimitiveTable and StackInterpreter>>functionPointerFor: primIdx inClass: theClass"
	| succeeded |
	<inline: true>
	primitiveFunctionPointer ~= 0 ifTrue:
		[self incPrimitiveCounter.
		 self isPrimitiveFunctionPointerAnIndex ifTrue:
			[self profileQuickPrimitive. ^self internalQuickPrimitiveResponse].
		 self profileSlowPrimitive.
		 self externalizeIPandSP.
		 succeeded := self slowPrimitiveResponse.
		 self internalizeIPandSP.
		 succeeded ifTrue:
			[self browserPluginReturnIfNeeded.
			^nil]].
	"if not primitive, or primitive failed, activate the method"
	^self internalActivateNewMethod! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 14:28'!
printProfilingResults

	<api>
	self cCode: 'unsigned int i;'.
	self cCode: 'printf("__VM_STATS__START__{ \
	#byteCodeCounter ->%llu. \
	#messageCounter -> %llu. \
	#primitiveCounter -> %llu. \
	#byteCode -> #( \
	",bCounter,mCounter,pCounter);'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",bArrayCounter[i])'.
	self  cCode: 'printf("). \
	#quickPrimitive -> #( \
	");'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",quickPrimitiveCounter[i])'.
	self cCode: 'printf("). \
	#slowPrimitive -> #( \
	");'.
	self cCode: 'for(i=0;i<255;i++) printf("%llu\n",slowPrimitiveCounter[i])'.
	self cCode: 'printf("). \
	}.__VM_STATS__END__ \
	");'.! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:28'!
profileQuickPrimitive

	<api>
	|index|
	index := (self cCoerce: primitiveFunctionPointer to: 'unsigned long').
	self cCode: 'quickPrimitiveCounter[index]++;'. "check if naming persists in translation"! !

!ExtendedProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 20:14'!
profileSlowPrimitive

	<api>
	|index|
	index := -1.
	1 to: MaxPrimitiveIndex do: [ :i | (primitiveTable at: i) == primitiveFunctionPointer ifTrue: [ index := i ]  ].
	self cCode: 'if(index > 0){slowPrimitiveCounter[index]++;}'. "check if naming persists in translation"! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:21'!
commonSend
	"Send a message, starting lookup with the receiver's class."
	"Assume: messageSelector and argumentCount have been set, and that 
	the receiver and arguments have been pushed onto the stack,"
	"Note: This method is inlined into the interpreter dispatch loop."
	<sharedCodeNamed: 'commonSend' inCase: #singleExtendedSendBytecode>
	self incMessageCounter.
	super commonSend.! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:21'!
fetchNextBytecode
	"This method fetches the next instruction (bytecode). Each bytecode method is responsible for fetching the next bytecode, preferably as early as possible to allow the memory system time to process the request before the next dispatch."
	
	self incByteCodeCounter.
	self cppIf: MULTIPLEBYTECODESETS
		ifTrue: [currentBytecode := self fetchByte + bytecodeSetSelector]
		ifFalse: [currentBytecode := self fetchByte]! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incByteCodeCounter

	<api>
	self cCode: 'bCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incMessageCounter
	<api>
	self cCode: 'mCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:20'!
incPrimitiveCounter
	<api>
	self cCode: 'pCounter++;'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:19'!
internalExecuteNewMethod
	"regardless of whether it succeeded or not count it since it is executed"
	| succeeded |
	<inline: true>
	primitiveFunctionPointer ~= 0 ifTrue:
		[self incPrimitiveCounter.self isPrimitiveFunctionPointerAnIndex ifTrue:
			[^self internalQuickPrimitiveResponse].
		 self externalizeIPandSP.
		 succeeded := self slowPrimitiveResponse.
		 self internalizeIPandSP.
		 succeeded ifTrue:
			[self browserPluginReturnIfNeeded.
			^nil]].
	"if not primitive, or primitive failed, activate the method"
	^self internalActivateNewMethod! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 11:22'!
primitiveQuit
	self printProfilingResults.
	super primitiveQuit.! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:08'!
print: s
	"For testing in Smalltalk, this method should be overridden in a subclass."
	<api>
	<var: #s type: #'char *'>
	self cCode: 'fputs(s, stdout)'! !

!ProfileStackInterpreter methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/12/2014 13:57'!
printProfilingResults
	<api>
	self cCode: 'printf("__VM_STATS__START__{ \
	#byteCodeCounter ->%llu. \
	#messageCounter -> %llu. \
	#primitiveCounter -> %llu. }.__VM_STATS__END__ \
	",bCounter,mCounter,pCounter);'.
	! !

StackUnixDebugConfig subclass: #ProfileUnixDebugConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:25'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:07'!
executableName
	^ 'ProfileStackDebugVM'! !

!ProfileUnixDebugConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/5/2014 10:06'!
interpreterClass
	^ ProfileStackInterpreter! !

StackUnixConfig subclass: #ProfileUnixConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

ProfileUnixConfig subclass: #ExtendedProfileUnixConfig
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 18:38'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

uint64_t bArrayCounter[255];
uint64_t quickPrimitiveCounter[520];
uint64_t slowPrimitiveCounter[576];

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:11'!
executableName
	^ 'ExtendedProfileStackVM'! !

!ExtendedProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:11'!
interpreterClass
	^ ExtendedProfileStackInterpreter! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
configH
	" right now its like  that "
	^ '
#ifndef __sq_config_h
#define __sq_config_h

/* explicit image width */

#define	HAVE_INTERP_H 1

/* package options */

#define	USE_X11 1
#define	USE_X11_GLX 1
/* #undef	USE_QUARTZ */
/* #undef	USE_QUARTZ_CGL */
/* #undef	USE_RFB */

/* libraries */

/* #undef	HAVE_LIBX11 */
/* #undef	HAVE_LIBXEXT */
#define	HAVE_LIBDL 1
/* #undef	HAVE_DYLD */
/* #undef	HAVE_LIBFFI */
/* #undef	HAVE_ICONV */

/* #undef	USE_AUDIO_NONE */
/* #undef	USE_AUDIO_SUN */
/* #undef	USE_AUDIO_NAS */
/* #undef	USE_AUDIO_OSS */
/* #undef	USE_AUDIO_MACOSX */
/* #undef	OSS_DEVICE */

/* header files */

#define	HAVE_UNISTD_H 1
/* #undef	NEED_GETHOSTNAME_P */

#define	HAVE_DIRENT_H 1
/* #undef	HAVE_SYS_NDIR_H */
/* #undef	HAVE_SYS_DIR_H */
/* #undef	HAVE_NDIR_H */
#define	HAVE_DLFCN_H 1
#define	HAVE_ICONV_H 1

#define	HAVE_SYS_TIME_H 1
#define	TIME_WITH_SYS_TIME 1

/* #undef	HAVE_SYS_FILIO_H */

/* #undef	HAVE_SYS_AUDIOIO_H */
/* #undef	HAVE_SUN_AUDIOIO_H */

#define	HAVE_PTY_H 1
/* #undef	HAVE_UTIL_H */
/* #undef	HAVE_LIBUTIL_H */
#define	HAVE_STROPTS_H 1

#define	HAVE_GL_GL_H 1
/* #undef	HAVE_OPENGL_GL_H */

/* #undef	NEED_SUNOS_H */

/* system calls/library functions */

#define	AT_EXIT atexit

#define	HAVE_TZSET 1

#define	HAVE_OPENPTY 1
/* #undef	HAVE_UNIX98_PTYS */

#define	HAVE_SNPRINTF 1
/* #undef	HAVE___SNPRINTF */

#define	HAVE_MMAP 1

/* #undef	HAVE_DYLD */

#define	HAVE_LANGINFO_CODESET 1

#define	HAVE_ALLOCA 1
#define	HAVE_ALLOCA_H 1

#define	HAVE_UNSETENV 1

#define	HAVE_NANOSLEEP 1

/* widths of primitive types */

#define	SIZEOF_INT 4
#define	SIZEOF_LONG 4
#define	SIZEOF_LONG_LONG 8
#define	SIZEOF_VOID_P 4

/* structures */

#define	HAVE_TM_GMTOFF 1
#define	HAVE_TIMEZONE 1

/* typedefs */

/* #undef	size_t */
/* #undef	socklen_t */

#define	squeakInt64 long long

/* architecture */

#define	OS_TYPE "unix"

#define	VM_HOST "i686-pc-linux-gnu"
#define	VM_HOST_CPU "i686"
/* #undef	VM_HOST_VENDOR */
#define	VM_HOST_OS "linux-gnu"
#define	VM_BUILD_STRING "Unix built on "__DATE__ " "__TIME__" Compiler: "__VERSION__

/* #undef	WORDS_BIGENDIAN */
/* #undef	DOUBLE_WORD_ALIGNMENT */

/* damage containment */

/* #undef	DARWIN */

#ifdef NEED_SUNOS_H
# include "sunos.h"
#endif

/* other configured variables */

#define SQ_VERSION "3.9a-7024"
#define VM_VERSION "3.9-7"
#define VM_MODULE_PREFIX ""
/* #undef VM_DLSYM_PREFIX */
#define VM_X11DIR ""

/* avoid dependencies on glibc2.3 */

#define HAVE_FEATURES_H 1

#if defined(HAVE_FEATURES_H)
# include "glibc.h"
#endif

#include <stdint.h>
uint64_t bCounter;
uint64_t mCounter;
uint64_t pCounter;

/* used by UUID plugin: */

#define HAVE_UUID_H 1

#endif /* __sq_config_h */
																																																																					
																																																																				'! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
executableName
	^ 'ProfileStackVM'! !

!ProfileUnixConfig methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/7/2014 16:07'!
interpreterClass
	^ ProfileStackInterpreter! !

Object subclass: #VanillaExperiment
	instanceVariableNames: 'runs averageRun'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

VanillaExperiment subclass: #ProfileExperiment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

ProfileExperiment subclass: #ExtendedProfileExperiment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

Object subclass: #VanillaExperimentRun
	instanceVariableNames: 'evalString realTime userTime systemTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

VanillaExperimentRun subclass: #ProfileExperimentRun
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

ProfileExperimentRun subclass: #ExtendedProfileExperimentRun
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UnixMetaStack-Profiling'!

!ExtendedProfileExperimentRun methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/13/2014 12:47'!
vm

	^ 'ExtendedProfileStackVM'! !

!ProfileExperimentRun methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/13/2014 12:46'!
vm

	^ 'ProfileStackVM'! !

!VanillaExperimentRun methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/13/2014 12:44'!
path

	^ '/home/art/Desktop/june/mStackVM/sources/results'! !

!VanillaExperimentRun methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 6/13/2014 12:43'!
vm

	^ 'StackVM'! !
